import axios from 'axios'
import Cookies from 'js-cookie'
import moment from 'moment-timezone'

import { URLS } from 'ferda_auction_js/Urls.js'
import { store } from 'ferda_auction_js/main.config'
import { get_axios_client } from 'ferda_auction_js/utils/axios.js'

axios.defaults.headers.common['X-CSRFToken'] = Cookies.get('csrftoken')

const AUCTION_ACCESS_TOKEN = 'auction_token'

const login_user = async() => {
    const axios_client = get_axios_client()
    const response = await axios_client.get(
        URLS.auction_login_ajax(),
    )
    sessionStorage.setItem(AUCTION_ACCESS_TOKEN, response.data.access_token)
}

const add_auth_to_axios_config = (config) => {
    config.headers.Authorization = `Bearer ${ sessionStorage.getItem(AUCTION_ACCESS_TOKEN) }`
    return config
}

let login_promise = null
const handle_parallel_login = () => {
    login_promise = login_user().finally(() => {
        login_promise = null
    })
}

export const use_auction_client = async(
    url,
    method,
    data = null,
    params = null,
    options = null,
    additional_status_codes = [],
) => {
    const axios_client = get_axios_client()

    axios_client.defaults.baseURL = store.getters['auction_settings/AUCTION_BASE_URL']
    axios_client.defaults.validateStatus = (status) =>
        (status >= 200 && status < 300) || additional_status_codes.includes(status)

    axios_client.interceptors.request.use(
        async(config) => {
            const access_token = sessionStorage.getItem(AUCTION_ACCESS_TOKEN)

            // Token missing completely, probably first attempt, we have to login
            if (access_token === null) {
                if (login_promise === null) {
                    handle_parallel_login()
                }
                await login_promise
                add_auth_to_axios_config(config)
                return config
            }

            // Decode expiration
            // atob is enough in this case as we decode just the base info
            // probably would break in case of custom payload
            const decoded_token = JSON.parse(atob(access_token.split('.')[0]))

            // Check if jwt is expired
            const TIME_ZONE = store.getters['settings/TIME_ZONE']
            const locale = localStorage
                .getItem(`preferences.${store.getters['general/user'].username}.selected_language`) || 'en'

            const expiration_dt = moment(decoded_token.exp * 1000).tz(TIME_ZONE).locale(locale)
            const now = moment().tz(TIME_ZONE).locale(locale)

            const is_expired = expiration_dt.isBefore(now)
            if (is_expired) {
                if (login_promise === null) {
                    handle_parallel_login()
                }
                await login_promise
            }

            add_auth_to_axios_config(config)
            return config
        },
    )
    // Dont append ? in case the request is without params
    const request_url = params === null ? url : url + `?${params.toString()}`
    let response
    try {
        response = await axios_client({
            ...(options ?? {}),
            url: request_url,
            method,
            data,
        })
    } catch {
        return null
    }
    return response
}
